package cz.muni.fi.pv168.invoicemanager.backend;

import cz.muni.fi.pv168.common.IllegalEntityException;
import cz.muni.fi.pv168.common.ValidationException;
import java.util.List;

/**
 * This service allows to manipulate with invoices.
 * 
 * @author Petr Adámek
 */
public interface InvoiceManager {
    
    /**
     * Stores new invoice into database. Id for the new invoice is automatically
     * generated and stored into id attribute.
     * 
     * @param invoice invoice to be created.
     * @throws IllegalArgumentException when invoice is null, or invoice has already 
     * assigned id.
     * @throws ValidationException when invoice breaks validation rules.
     * @throws IllegalEntityException when invoice already have asigned id.
     * @throws ServiceFailureException when db operation fails.
     */
    void createInvoice(Invoice invoice) throws ServiceFailureException, ValidationException, IllegalEntityException;
    
    /**
     * Returns invoice with given id.
     * 
     * @param id primary key of requested invoice.
     * @return invoice with given id or null if such invoice does not exist.
     * @throws IllegalArgumentException when given id is null.
     * @throws ServiceFailureException when db operation fails.
     */
    Invoice getInvoice(Long id) throws ServiceFailureException;
    
    /**
     * Updates invoice in database.
     * 
     * @param invoice updated invoice to be stored into database.
     * @throws IllegalArgumentException when invoice is null, or invoice has null id.
     * @throws ValidationException when invoice breaks validation rules.
     * @throws IllegalEntityException when invoice already have not asigned id 
     * or it doesn not exist in database.
     * @throws ServiceFailureException when db operation fails.
     */
    void updateInvoice(Invoice invoice) throws ServiceFailureException, ValidationException, IllegalEntityException;
    
    /**
     * Deletes invoice from database. 
     * 
     * @param invoice invoice to be deleted from db.
     * @throws IllegalArgumentException when invoice is null, or invoice has null id.
     * @throws IllegalEntityException when invoice already have not asigned id 
     * or it doesn not exist in database.
     * @throws ServiceFailureException when db operation fails.
     */
    void deleteInvoice(Invoice invoice) throws ServiceFailureException, IllegalEntityException;
    
    /**
     * Returns list of all invoices in the database.
     * 
     * @return list of all invoices in database.
     * @throws ServiceFailureException when db operation fails.
     */
    List<Invoice> findAllInvoices() throws ServiceFailureException;
    
}
